@@ -87,7 +87,7 @@ gint                    g_atomic_int_exchange_and_add         (volatile gint  *a
 
 G_END_DECLS
 
-#if defined(G_ATOMIC_LOCK_FREE)
+#if defined(G_ATOMIC_LOCK_FREE) && defined(__GCC_HAVE_SYNC_COMPARE_AND_SWAP_4)
 
 /* We prefer the new C11-style atomic extension of GCC if available */
 #if defined(__ATOMIC_SEQ_CST)
@@ -407,7 +407,7 @@ G_END_DECLS
 
 #endif /* !defined(__ATOMIC_SEQ_CST) */
 
-#else /* defined(G_ATOMIC_LOCK_FREE) */
+#else /* defined(G_ATOMIC_LOCK_FREE) && defined(__GCC_HAVE_SYNC_COMPARE_AND_SWAP_4) */
 
 #define g_atomic_int_get(atomic) \
   (g_atomic_int_get ((gint *) (atomic)))
